#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   .-.                              _____                                  __
#   /v\    L   I   N   U   X       / ____/__   ___   ___   ____ ___   ___  / /_  __  __
#  // \\                          / / __/ _ \/ __ \/ __ `/ ___/ __ `/ __ \/ __ \/ / / /
# /(   )\                        / /_/ /  __/ /_/ / /_/ / /  / /_/ / /_/ / / / / /_/ /
#  ^^-^^                         \____/\___/\____/\__, /_/   \__,_/ .___/_/ /_/\__, /
#                                                /____/          /_/          /____/

from datetime import datetime as dt
import argparse
import os
import csv



def main(in_CSV, start_field, end_field):

    # Define a new name for our output CSV by adding "_dur"
    # to the end, but not before the .csv extension. We
    # separate out the extension, then append it back in.
    in_path_no_ext, ext = os.path.splitext(in_CSV)
    new_path_no_ext = in_path_no_ext + "_dur"
    out_path = new_path_no_ext + ext

    # Open and read the CSV file.
    with open(in_CSV) as in_file:

        # Use A DictReader to read row by row.
        d_reader = csv.DictReader(
            in_file,
            delimiter=',',
            quotechar='"'
        )
        
        # Get the existing field names in the input file.
        field_names = d_reader.fieldnames
        
        # Add our new field to those.
        field_names.append("seconds")

        # Open the new, output CSV file.
        with open(out_path, 'w', newline='') as out_file:

            # Use a DictWriter to write new values to the output file.
            # Define its headers and write them, too.
            d_writer = csv.DictWriter(
                out_file, 
                delimiter=',',
                quotechar='"',
                fieldnames=field_names
            )
            d_writer.writeheader()

            # Go through the input CSV row by row.
            for row in d_reader: 

                # Convert the timestamps in the row to ISO format by replacing
                # the space with a capital T.
                start_ISO = row[start_field].replace(" ", "T")
                end_ISO = row[end_field].replace(" ", "T")

                # Save the timestamps as Python datetime objects.
                start_datetime = dt.fromisoformat(start_ISO)
                end_datetime = dt.fromisoformat(end_ISO)

                # Calculate the time difference by subtracting the end
                # datetime object from the start datetime object.
                time_delta = end_datetime - start_datetime

                # Get the time duration expressed in seconds using datetime's
                # total_seconds method.
                sec_count = time_delta.total_seconds()

                # Add this value to the set of values defined in `row`
                # as the value for the header "seconds".
                row["seconds"] = sec_count

                # Write the values defined in `row`, which now include our new
                # `seconds` value, to a row of the output CSV.
                d_writer.writerow(row)



if __name__ == '__main__':
    
    # Usage messages, and parse command line arguments.
    parser = argparse.ArgumentParser(
        description="A brief script to calculate trip duration in Divvy bikeshare CSV data." 
    )
    parser.add_argument("INCSV", 
        help="CSV file containing bikeshare trips."
    )
    parser.add_argument("STARTFIELD", 
        help="Field in the input CSV with a time stamp for the beginning "
        "of the trip. Expecting `YYYY-MM-DD HH:MM:SS` format."
    )
    parser.add_argument("ENDFIELD", 
        help="Field in the input CSV with a time stamp for the end "
        "of the trip. Expecting `YYYY-MM-DD HH:MM:SS` format."
    )

    args = parser.parse_args()

    main(
        args.INCSV,
        args.STARTFIELD,
        args.ENDFIELD
    )
